
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "output.h"

void DumpOSVersion( Output& output );
void DumpD3DCAPS8( Output& output );
void DumpD3DCAPS9( Output& output );
void DumpGLExtensions( Output& output );

#ifdef _DEBUG
int main()
#else
int mainCRTStartup()
#endif
{
    {
	static const char info[] =
	    "Capability Dumper, Version 2.0.1\r\n"
	    "Copyright (c) 2004 Chris Dragan\r\n"
	    "All Rights Reserved\r\n"
	    "Homepage: http://d3dcaps.chris.dragan.name/\r\n"
	    "\r\n"
	    "This program will dump:\r\n"
	    " - D3DCAPS - Direct3D capabilities of graphics adapters\r\n"
	    " - D3DFMTs - Direct3D surface formats of graphics adapters\r\n"
	    " - OpenGL Extensions of the system default graphics adapter\r\n"
	    "\r\n";
	DWORD num_writ = 0;
	WriteFile( GetStdHandle( STD_OUTPUT_HANDLE ), info, sizeof(info)-1, &num_writ, NULL );
    }

    Output output( "devcaps.txt" );
    DumpOSVersion( output );
    DumpD3DCAPS8( output );
    DumpD3DCAPS9( output );
    DumpGLExtensions( output );

    {
	static const char info[] =
	    "The capabilities were saved in file devcaps.txt\r\n"
	    "Please submit them to d3dcaps@chris.dragan.name\r\n"
	    "\r\n";
	DWORD num_writ = 0;
	WriteFile( GetStdHandle( STD_OUTPUT_HANDLE ), info, sizeof(info)-1, &num_writ, NULL );
    }
    ExitProcess( 0 );
    return 0;
}

void ClearMem( void* ptr, size_t size )
{
    char* p = static_cast<char*>( ptr );
    while ( size-- ) *(p++) = 0;
}

void DumpOSVersion( Output& output )
{
    // Get Windows version
    OSVERSIONINFO osver;
    ClearMem( &osver, sizeof(osver) );
    osver.dwOSVersionInfoSize = sizeof(osver);
    GetVersionEx( &osver );
    const char* os = "";
    const int version = osver.dwMajorVersion * 100 + osver.dwMinorVersion;
    switch ( osver.dwPlatformId ) {

	case VER_PLATFORM_WIN32s:
	    os = "Win31 with Win32s";
	    break;

	case VER_PLATFORM_WIN32_WINDOWS:
	    switch ( version ) {
		case 400: os = "Win95"; break;
		case 410: os = "Win98"; break;
		case 490: os = "WinME"; break;
		default: os = "Win?? (95-class)"; break;
	    }
	    break;

	case VER_PLATFORM_WIN32_NT:
	    switch ( version ) {
		case 351: os = "WinNT 3.51"; break;
		case 400: os = "WinNT 4.0"; break;
		case 500: os = "Win2K"; break;
		case 501: os = "WinXP"; break;
		default: os = "Win?? (NT-class)"; break;
	    }
	    break;
    }

    // Get DirectX version if available
    char dx[ 64 ] = "";
    HKEY key;
    if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\DirectX", 0, KEY_READ, &key ) == ERROR_SUCCESS ) {
	DWORD size = sizeof(dx)-1;
	RegQueryValueEx( key, "Version", 0, 0, (LPBYTE)dx, &size );
	dx[ size ] = 0;
	RegCloseKey( key );
    }

    // Display version
    char buf[256];
    wsprintf( buf, "%s, Version %d.%d.%d %s, DirectX Version %s",
	os,
	osver.dwMajorVersion,
	osver.dwMinorVersion,
	osver.dwBuildNumber,
	((osver.szCSDVersion!=NULL) ? osver.szCSDVersion : ""),
	dx );
    output.Write( "OSVERSION", buf );
}
